package com.zanxiang.sdk.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.zanxiang.common.domain.ResultMap;
import com.zanxiang.common.enums.OrderStateEnum;
import com.zanxiang.common.enums.PayDeviceEnum;
import com.zanxiang.common.exception.BaseException;
import com.zanxiang.common.utils.SpringUtils;
import com.zanxiang.module.util.JsonUtil;
import com.zanxiang.module.util.pojo.ResultVO;
import com.zanxiang.module.web.util.IpUtil;
import com.zanxiang.mybatis.entity.Order;
import com.zanxiang.mybatis.entity.OrderPayParam;
import com.zanxiang.sdk.domain.bo.ProductPayParamBO;
import com.zanxiang.sdk.domain.dto.GamePayWayDTO;
import com.zanxiang.sdk.domain.dto.PayApplicationDTO;
import com.zanxiang.sdk.domain.params.ProductPayParam;
import com.zanxiang.sdk.domain.params.UserData;
import com.zanxiang.sdk.domain.vo.PayParamVO;
import com.zanxiang.sdk.enums.PayTypeEnum;
import com.zanxiang.sdk.service.*;
import com.zanxiang.sdk.service.pay.PayBaseService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author : lingfeng
 * @time : 2022-09-28
 * @description : 订单支付
 */
@Slf4j
@Service
public class OrderPayServiceImpl implements IOrderPayService {

    @Autowired
    public IOrderService orderService;

    @Autowired
    public IGamePayWayService gamePayWayService;

    @Autowired
    private IOrderPayParamService orderPayParamService;

    @Autowired
    private IPayApplicationService payApplicationService;

    @Override
    public PayParamVO getPayParam(String code, String orderId, HttpServletRequest request) {
        Order order = orderService.getOne(new LambdaQueryWrapper<Order>()
                .eq(Order::getOrderId, orderId));
        if (order == null) {
            throw new BaseException("参数错误, 订单信息不存在");
        }
        if (Objects.equals(order.getStatus(), OrderStateEnum.CANCEL_PAY.getCode())) {
            throw new BaseException("订单已关闭, 请回到游戏重新下单");
        }
        if (Objects.equals(order.getStatus(), OrderStateEnum.SUCCESS_PAY.getCode())) {
            throw new BaseException("订单已支付, 请勿重复支付");
        }
        OrderPayParam orderPayParam = orderPayParamService.getOne(new LambdaQueryWrapper<OrderPayParam>()
                .eq(OrderPayParam::getOrderId, order.getOrderId()));
        if (orderPayParam != null) {
            return PayParamVO.builder()
                    .orderId(orderPayParam.getOrderId())
                    .appId(orderPayParam.getAppId())
                    .payParam(orderPayParam.getPayParam())
                    .build();
        }
        //创建支付参数
        ProductPayParamBO bo = ProductPayParamBO.builder()
                .gameId(order.getGameId())
                .userId(order.getUserId())
                .payDevice(order.getPayDeviceId())
                .deviceSystem(order.getDeviceSystem())
                .spbillCreateIp(IpUtil.getRealIp(request))
                .outTradeNo(order.getOrderId())
                .payWay(order.getPayWayId())
                .code(code)
                .build();
        //调起支付
        PayTypeEnum payTypeEnum = PayTypeEnum.getByPayType(bo.getPayWay().intValue());
        PayBaseService service = SpringUtils.getBean(payTypeEnum.getClazz());
        service.payCreate(bo);
        //重新查询
        orderPayParam = orderPayParamService.getOne(new LambdaQueryWrapper<OrderPayParam>()
                .eq(OrderPayParam::getOrderId, order.getOrderId()));
        if (orderPayParam == null) {
            throw new BaseException("支付参数获取失败");
        }
        return PayParamVO.builder()
                .orderId(orderPayParam.getOrderId())
                .appId(orderPayParam.getAppId())
                .payParam(orderPayParam.getPayParam())
                .build();
    }

    @Override
    public Integer getAppletPayDevice(UserData userData) {
        return 5;
    }

    @Override
    public Map<String, Object> payCreate(ProductPayParam product, UserData userData) {
        //创建订单
        orderService.createOrder(product, userData);
        //判断游戏类型
        if (Objects.equals(PayDeviceEnum.APPLET_PAY.getPayDeviceId(), product.getPayDevice())
                || Objects.equals(PayDeviceEnum.CUSTOM_PAY.getPayDeviceId(), product.getPayDevice())) {
            //查询支付配置
            GamePayWayDTO gamePayWayDTO = gamePayWayService.getGamePayWay(userData.getGameId(), product.getPayWay(), product.getPayDevice());
            //查询支付应用信息
            PayApplicationDTO payApplicationDTO = payApplicationService.getPayApplicationByPayBoxId(gamePayWayDTO.getPayBoxId());
            //构造参数信息
            Map<Object, Object> paramMap = new HashMap<>(4);
            paramMap.put("appId", payApplicationDTO.getAppId());
            paramMap.put("path", payApplicationDTO.getPath());
            paramMap.put("orderId", product.getOrderId());
            paramMap.put("amount", product.getAmount());
            log.error("下单参数返回, paramMap : {}", JsonUtil.toString(paramMap));
            return ResultMap.ok(paramMap);
        }
        //创建支付参数
        ProductPayParamBO bo = new ProductPayParamBO();
        bo.setGameId(userData.getGameId());
        bo.setUserId(userData.getUserId());
        bo.setPayDevice(product.getPayDevice());
        bo.setDeviceSystem(userData.getDeviceSystem());
        bo.setSpbillCreateIp(userData.getIp());
        bo.setOutTradeNo(product.getOrderId());
        bo.setPayWay(product.getPayWay());
        bo.setCode(product.getCode());
        //调起支付
        PayTypeEnum payTypeEnum = PayTypeEnum.getByPayType(bo.getPayWay().intValue());
        PayBaseService service = SpringUtils.getBean(payTypeEnum.getClazz());
        return service.payCreate(bo);
    }

    @Override
    public String notify(HttpServletRequest request, HttpServletResponse response, Integer payType) throws IOException {
        PayTypeEnum payTypeEnum = PayTypeEnum.getByPayType(payType);
        PayBaseService service = SpringUtils.getBean(payTypeEnum.getClazz());
        return service.notify(request, response);
    }

    @Override
    public ResultVO synNotify(HttpServletRequest request, Integer payType) {
        PayTypeEnum payTypeEnum = PayTypeEnum.getByPayType(payType);
        PayBaseService service = SpringUtils.getBean(payTypeEnum.getClazz());
        return service.synNotify(request);
    }

    @Override
    public Boolean payResult(String orderId) {
        Order order = orderService.getOne(new LambdaQueryWrapper<Order>()
                .eq(Order::getOrderId, orderId));
        if (order == null) {
            throw new BaseException("参数错误, 订单信息不存在");
        }
        if (Objects.equals(order.getStatus(), OrderStateEnum.SUCCESS_PAY.getCode())) {
            return Boolean.TRUE;
        }
        return Boolean.FALSE;
    }
}
